@@ -50,6 +50,7 @@ const char *attribute_map[] = {
 	CM_SUBMIT_REQ_EMAIL_ENV, CM_DBUS_PROP_TEMPLATE_EMAIL,
 	CM_SUBMIT_REQ_IP_ADDRESS_ENV, CM_DBUS_PROP_TEMPLATE_IP_ADDRESS,
 	CM_SUBMIT_PROFILE_ENV, CM_DBUS_PROP_TEMPLATE_PROFILE,
+	CM_SUBMIT_ISSUER_ENV, CM_DBUS_PROP_TEMPLATE_ISSUER,
 	NULL,
 };
 
@@ -274,6 +275,7 @@ parse_json_cert_list(void *parent, struct cm_json *nickcerts)
 			     i++) {
 				c = talloc_ptrtype(ret, c);
 				if (c != NULL) {
+					memset(c, 0, sizeof(*c));
 					cert = cm_json_n(nickcerts, i);
 					if (cm_json_type(cert) != cm_json_type_object) {
 						continue;
