@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2011,2012,2013,2014 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2012,2013,2014,2015 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +37,8 @@
 
 #include <talloc.h>
 
+#include <popt.h>
+
 #include "log.h"
 #include "prefs.h"
 #include "store.h"
@@ -55,33 +57,9 @@
 #define _(_text) (_text)
 #endif
 
-#define IPACONFIG "/etc/ipa/default.conf"
-#define IPASECTION "dogtag"
-
-static void
-help(const char *cmd)
-{
-	fprintf(stderr,
-		"Usage: %s -E EE-URL -A AGENT-URL [options]\n"
-		"Options:\n"
-		"\t[-d dbdir]\n"
-		"\t[-n nickname]\n"
-		"\t[-i cainfo]\n"
-		"\t[-C capath]\n"
-		"\t[-c certfile]\n"
-		"\t[-k keyfile]\n"
-		"\t[-p pinfile]\n"
-		"\t[-P pin]\n"
-		"\t[-s serial (hex)]\n"
-		"\t[-D serial (decimal)]\n"
-		"\t[-S state]\n"
-		"\t[-T profile]\n"
-		"\t[-v]\n"
-		"\t[-N]\n"
-		"\t[-V dogtag_version]\n"
-		"\t[csrfile]\n",
-		strchr(cmd, '/') ? strrchr(cmd, '/') + 1 : cmd);
-}
+#ifdef DOGTAG_IPA_RENEW_AGENT
+#include "dogtag-ipa.h"
+#endif
 
 static char *
 statevar(const char *state, const char *what)
@@ -133,29 +111,75 @@ serial_hex_from_cert(const char *cert)
 }
 
 int
-main(int argc, char **argv)
+main(int argc, const char **argv)
 {
 	const char *eeurl = NULL, *agenturl = NULL, *url = NULL, *url2 = NULL;
 	const char *ssldir = NULL, *cainfo = NULL, *capath = NULL;
 	const char *sslcert = NULL, *sslkey = NULL;
 	const char *sslpin = NULL, *sslpinfile = NULL;
-	const char *host = NULL, *csr = NULL, *serial = NULL, *template = NULL;
-	const char *dogtag_version = NULL;
-	char *ipaconfig = NULL, *savedstate = NULL;
+	const char *csr = NULL, *serial = NULL, *template = NULL;
+	const char *uid = NULL, *pwd = NULL, *pwdfile = NULL;
+	const char *udn = NULL, *pin = NULL, *pinfile = NULL;
+	char *poptarg;
+	struct {
+		char *name;
+		char *value;
+	} *aoptions = NULL, *soptions = NULL;
+	size_t num_aoptions = 0, num_soptions = 0, j;
+	char *savedstate = NULL;
 	char *p, *q, *params = NULL, *params2 = NULL;
 	const char *lasturl = NULL, *lastparams = NULL;
 	const char *tmp = NULL, *results = NULL;
 	struct cm_submit_h_context *hctx;
 	void *ctx;
 	int c, verbose = 0, force_new = 0, force_renew = 0, i;
+#ifdef DOGTAG_IPA_RENEW_AGENT
+	const char *host = NULL, *dogtag_version = NULL;
 	int eeport, agentport;
-	enum { op_none, op_submit, op_check, op_approve, op_retrieve } op = op_none;
-	dbus_bool_t can_agent, use_agent, missing_args = FALSE;
+#endif
+	enum { op_none, op_submit, op_check, op_approve, op_retrieve, op_profiles } op = op_submit;
+	dbus_bool_t can_agent, use_agent_approval = FALSE, missing_args = FALSE;
+	dbus_bool_t use_agent_submission = FALSE;
 	struct dogtag_default **defaults;
 	enum cm_external_status ret;
 	NSSInitContext *nctx;
 	const char *es;
-	const char *mode = CM_OP_SUBMIT;
+	const char *mode = CM_OP_SUBMIT, *csrfile;
+	poptContext pctx;
+	const struct poptOption popts[] = {
+		{"ee-url", 'E', POPT_ARG_STRING, &eeurl, 0, "end-entity services location", "URL"},
+		{"agent-url", 'A', POPT_ARG_STRING, &agenturl, 0, "agent services location", "URL"},
+		{"cafile", 'i', POPT_ARG_STRING, &cainfo, 0, NULL, "FILENAME"},
+		{"capath", 'C', POPT_ARG_STRING, &capath, 0, NULL, "DIRECTORY"},
+		{"dbdir", 'd', POPT_ARG_STRING, &ssldir, 0, "database containing agent or client creds", "DIRECTORY"},
+		{"nickname", 'n', POPT_ARG_STRING, &sslcert, 0, "nickname of agent or client creds", "NAME"},
+		{"certfile", 'c', POPT_ARG_STRING, &sslcert, 0, "agent or client certificate", "FILENAME"},
+		{"keyfile", 'k', POPT_ARG_STRING, &sslkey, 0, "agent or client key", "FILENAME"},
+		{"sslpinfile", 'p', POPT_ARG_STRING, &sslpinfile, 0, "agent or client key pinfile", "FILENAME"},
+		{"sslpin", 'P', POPT_ARG_STRING, &sslpin, 0, "agent or client key pin", NULL},
+		{"hex-serial", 's', POPT_ARG_STRING, NULL, 's', "request renewal for certificate by serial number (hexadecimal)", "NUMBER"},
+		{"serial", 'D', POPT_ARG_STRING, &serial, 'D', "request renewal for certificate by serial number", "NUMBER"},
+		{"submit-option", 'o', POPT_ARG_STRING, NULL, 'o', "key-value pair to send to server", NULL},
+		{"approval-option", 'O', POPT_ARG_STRING, NULL, 'O', "key-value pair to set in certificate", NULL},
+		{"profile", 'T', POPT_ARG_STRING, &template, 0, "enrollment profile", "NAME"},
+		{"profile-list", 't', POPT_ARG_NONE, NULL, 't', "list enrollment profiles", NULL},
+		{"state", 'S', POPT_ARG_STRING, &savedstate, 0, "previously-provided state data", "STATE-VALUE"},
+#ifdef DOGTAG_IPA_RENEW_AGENT
+		{"dogtag-version", 'V', POPT_ARG_STRING, &dogtag_version, 'V', NULL, "NUMBER"},
+#endif
+		{"force-new", 'N', POPT_ARG_NONE, NULL, 'N', "prefer to obtain a new certificate", NULL},
+		{"force-renew", 'R', POPT_ARG_NONE, NULL, 'R', "prefer to renew a certificate", NULL},
+		{"agent-submit", 'a', POPT_ARG_NONE, NULL, 'a', "submit enrollment or renewal request using agent or client creds", NULL},
+		{"uid", 'u', POPT_ARG_STRING, &uid, 0, "submit enrollment or renewal request using user name", "USERNAME"},
+		{"udn", 'U', POPT_ARG_STRING, &udn, 0, "submit enrollment or renewal request using user DN", "USERDN"},
+		{"userpwd", 'W', POPT_ARG_STRING, &pwd, 0, "submit password with enrollment or renewal request", NULL},
+		{"userpwdfile", 'w', POPT_ARG_STRING, &pwdfile, 0, "submit password from file with enrollment or renewal request", "FILENAME"},
+		{"userpin", 'Y', POPT_ARG_STRING, &pin, 0, "submit pin with enrollment or renewal request", NULL},
+		{"userpinfile", 'y', POPT_ARG_STRING, &pinfile, 0, "submit pin from file with enrollment or renewal request", "FILENAME"},
+		{"verbose", 'v', POPT_ARG_NONE, NULL, 'v', NULL, NULL},
+		POPT_AUTOHELP
+		POPT_TABLEEND
+	};
 
 	if (getenv(CM_SUBMIT_OPERATION_ENV) != NULL) {
 		mode = getenv(CM_SUBMIT_OPERATION_ENV);
@@ -164,8 +188,16 @@ main(int argc, char **argv)
 	    (strcasecmp(mode, CM_OP_POLL) == 0)) {
 		/* fall through */
 	} else
+	if (strcasecmp(mode, CM_OP_FETCH_PROFILES) == 0) {
+		op = op_profiles;
+	} else
 	if (strcasecmp(mode, CM_OP_IDENTIFY) == 0) {
+#ifdef DOGTAG_IPA_RENEW_AGENT
+		printf("Dogtag (IPA,renew,agent) (%s %s)\n", PACKAGE_NAME,
+		       PACKAGE_VERSION);
+#else
 		printf("Dogtag (%s %s)\n", PACKAGE_NAME, PACKAGE_VERSION);
+#endif
 		return 0;
 	} else {
 		/* unsupported request */
@@ -178,54 +210,73 @@ main(int argc, char **argv)
 
 	savedstate = getenv(CM_SUBMIT_COOKIE_ENV);
 
-	while ((c = getopt(argc, argv, "E:A:d:n:i:C:c:k:p:P:s:D:S:T:vV:NR")) != -1) {
+	pctx = poptGetContext(argv[0], argc, argv, popts, 0);
+	if (pctx == NULL) {
+		return CM_SUBMIT_STATUS_UNCONFIGURED;
+	}
+	poptSetOtherOptionHelp(pctx, "[options] -E EE-URL -A AGENT-URL [csrfile]");
+	while ((c = poptGetNextOpt(pctx)) > 0) {
 		switch (c) {
-		case 'E':
-			eeurl = optarg;
-			break;
-		case 'A':
-			agenturl = optarg;
-			break;
-		case 'd':
-			ssldir = optarg;
-			break;
-		case 'i':
-			cainfo = optarg;
-			break;
-		case 'C':
-			capath = optarg;
-			break;
-		case 'c':
-		case 'n':
-			sslcert = optarg;
-			break;
-		case 'k':
-			sslkey = optarg;
-			break;
-		case 'p':
-			sslpinfile = optarg;
-			break;
-		case 'P':
-			sslpin = optarg;
-			break;
-		case 'D':
-			serial = optarg;
-			break;
 		case 's':
-			serial = util_dec_from_hex(optarg);
+			serial = util_dec_from_hex(poptGetOptArg(pctx));
 			break;
-		case 'S':
-			savedstate = optarg;
+		case 'O':
+			poptarg = poptGetOptArg(pctx);
+			if (strchr(poptarg, '=') == NULL) {
+				printf(_("Profile params (-O) must be in the form of param=value.\n"));
+				poptPrintUsage(pctx, stdout, 0);
+				return CM_SUBMIT_STATUS_UNCONFIGURED;
+			}
+			aoptions = realloc(aoptions,
+					   ++num_aoptions * sizeof(*aoptions));
+			if (aoptions == NULL) {
+				printf(_("Out of memory.\n"));
+				return CM_SUBMIT_STATUS_UNCONFIGURED;
+			}
+			p = strdup(poptarg);
+			if (p == NULL) {
+				printf(_("Out of memory.\n"));
+				return CM_SUBMIT_STATUS_UNCONFIGURED;
+			}
+			i = strcspn(p, "=");
+			aoptions[num_aoptions - 1].name = p;
+			p[i] = '\0';
+			aoptions[num_aoptions - 1].value = p + i + 1;
 			break;
-		case 'T':
-			template = optarg;
+		case 'o':
+			poptarg = poptGetOptArg(pctx);
+			if (strchr(poptarg, '=') == NULL) {
+				printf(_("Submit params (-o) must be in the form of param=value.\n"));
+				poptPrintUsage(pctx, stdout, 0);
+				return CM_SUBMIT_STATUS_UNCONFIGURED;
+			}
+			soptions = realloc(soptions,
+					   ++num_soptions * sizeof(*soptions));
+			if (soptions == NULL) {
+				printf(_("Out of memory.\n"));
+				return CM_SUBMIT_STATUS_UNCONFIGURED;
+			}
+			p = strdup(poptarg);
+			if (p == NULL) {
+				printf(_("Out of memory.\n"));
+				return CM_SUBMIT_STATUS_UNCONFIGURED;
+			}
+			i = strcspn(p, "=");
+			soptions[num_soptions - 1].name = p;
+			p[i] = '\0';
+			soptions[num_soptions - 1].value = p + i + 1;
+			break;
+		case 't':
+			op = op_profiles;
 			break;
 		case 'v':
 			verbose++;
 			break;
+#ifdef DOGTAG_IPA_RENEW_AGENT
 		case 'V':
-			dogtag_version = optarg;
+			dogtag_version = poptGetOptArg(pctx);
 			break;
+#endif
 		case 'N':
 			force_new++;
 			force_renew = 0;
@@ -234,14 +285,19 @@ main(int argc, char **argv)
 			force_renew++;
 			force_new = 0;
 			break;
-		default:
-			help(argv[0]);
-			return CM_SUBMIT_STATUS_UNCONFIGURED;
+		case 'a':
+			use_agent_submission = TRUE;
 			break;
 		}
 	}
+	if (c != -1) {
+		poptPrintUsage(pctx, stdout, 0);
+		return CM_SUBMIT_STATUS_UNCONFIGURED;
+	}
 
 	umask(S_IRWXG | S_IRWXO);
+	cm_log_set_method(cm_log_stderr);
+	cm_log_set_level(verbose);
 
 	nctx = NSS_InitContext(CM_DEFAULT_CERT_STORAGE_LOCATION,
 			       NULL, NULL, NULL, NULL,
@@ -261,21 +317,8 @@ main(int argc, char **argv)
 
 	ctx = talloc_new(NULL);
 
-	ipaconfig = read_config_file(IPACONFIG);
-	if (ipaconfig != NULL) {
-		host = get_config_entry(ipaconfig,
-					"global",
-					"host");
-		if (dogtag_version == NULL) {
-			dogtag_version = get_config_entry(ipaconfig,
-							  "global",
-							  "dogtag_version");
-		}
-	} else {
-		host = NULL;
-		dogtag_version = NULL;
-	}
-
+#ifdef DOGTAG_IPA_RENEW_AGENT
+	cm_dogtag_ipa_hostver(&host, &dogtag_version);
 	if ((dogtag_version != NULL) && (atof(dogtag_version) >= 10)) {
 		eeport = 8080;
 		agentport = 8443;
@@ -283,7 +326,6 @@ main(int argc, char **argv)
 		eeport = 9180;
 		agentport = 9443;
 	}
-
 	if (eeurl == NULL) {
 		eeurl = cm_prefs_dogtag_ee_url();
 		if ((eeurl == NULL) && (host != NULL)) {
@@ -300,6 +342,8 @@ main(int argc, char **argv)
 						   host, agentport);
 		}
 	}
+#endif
+
 	if (template == NULL) {
 		template = getenv(CM_SUBMIT_PROFILE_ENV);
 		if (template == NULL) {
@@ -341,6 +385,7 @@ main(int argc, char **argv)
 	if ((sslpinfile == NULL) && (sslpin == NULL)) {
 		sslpinfile = cm_prefs_dogtag_sslpinfile();
 	}
+#ifdef DOGTAG_IPA_RENEW_AGENT
 	if ((cainfo == NULL) &&
 	    (capath == NULL) &&
 	    (ssldir == NULL) &&
@@ -353,11 +398,22 @@ main(int argc, char **argv)
 		sslcert = "ipaCert";
 		sslpinfile = "/etc/httpd/alias/pwdfile.txt";
 	}
+#endif
 	if ((sslcert != NULL) && (strlen(sslcert) > 0)) {
 		can_agent = TRUE;
 	} else {
 		can_agent = FALSE;
 	}
+	if (use_agent_approval && !can_agent) {
+		printf(_("No agent credentials specified, and no "
+			 "default known.\n"));
+		missing_args = TRUE;
+	}
+	if (use_agent_submission && !can_agent) {
+		printf(_("No agent credentials specified, and no "
+			 "default known.\n"));
+		missing_args = TRUE;
+	}
 	if (force_renew && (serial == NULL)) {
 		printf(_("Requested renewal, but no serial number provided.\n"));
 		missing_args = TRUE;
@@ -366,16 +422,30 @@ main(int argc, char **argv)
 		printf(_("No end-entity URL (-E) given, and no default known.\n"));
 		missing_args = TRUE;
 	}
+#ifdef DOGTAG_IPA_RENEW_AGENT
 	if (agenturl == NULL) {
 		printf(_("No agent URL (-A) given, and no default known.\n"));
 		missing_args = TRUE;
 	}
+#endif
 	if (template == NULL) {
 		printf(_("No profile/template (-T) given, and no default known.\n"));
 		missing_args = TRUE;
 	}
+	if (aoptions != NULL) {
+		if (agenturl == NULL) {
+			printf(_("No agent URL (-A) given, and no default "
+				 "known.\n"));
+			missing_args = TRUE;
+		}
+		if (!can_agent) {
+			printf(_("No agent credentials specified, and no "
+				 "default known.\n"));
+			missing_args = TRUE;
+		}
+	}
 	if (missing_args) {
-		help(argv[0]);
+		poptPrintUsage(pctx, stdout, 0);
 		return CM_SUBMIT_STATUS_UNCONFIGURED;
 	}
 	if (NSS_ShutdownContext(nctx) != SECSuccess) {
@@ -384,7 +454,6 @@ main(int argc, char **argv)
 	}
 
 	/* Figure out where we are in the multi-step process. */
-	op = op_none;
 	if ((savedstate != NULL) &&
 	    ((p = statevar(savedstate, "state")) != NULL) &&
 	    ((q = statevar(savedstate, "requestId")) != NULL)) {
@@ -401,10 +470,17 @@ main(int argc, char **argv)
 		}
 		params = talloc_asprintf(ctx, "requestId=%s", q);
 	} else {
-		op = op_submit;
 		params = "";
 	}
 
+	/* Read the client password and/or PIN, if we need to. */
+	if ((pwdfile != NULL) && (pwd == NULL)) {
+		pwd = cm_submit_u_from_file(pwdfile);
+	}
+	if ((pinfile != NULL) && (pin == NULL)) {
+		pin = cm_submit_u_from_file(pinfile);
+	}
+
 	/* Figure out which form and arguments to use. */
 	switch (op) {
 	case op_none:
@@ -412,7 +488,10 @@ main(int argc, char **argv)
 		return CM_SUBMIT_STATUS_UNCONFIGURED;
 		break;
 	case op_submit:
-		url = talloc_asprintf(ctx, "%s/profileSubmit", eeurl);
+		url = talloc_asprintf(ctx, "%s/%s", eeurl,
+				      use_agent_submission ?
+				      "profileSubmitSSLClient" :
+				      "profileSubmit");
 		template = cm_submit_u_url_encode(template);
 		if ((serial != NULL) && (strlen(serial) > 0) && !force_new) {
 			/* Renew-by-serial. */
@@ -428,15 +507,24 @@ main(int argc, char **argv)
 			/* Fresh enrollment.  Read the CSR from the
 			 * environment, or from the command-line, that we're
 			 * going to submit for signing. */
-			csr = getenv(CM_SUBMIT_CSR_ENV);
-			if (csr == NULL) {
-				csr = cm_submit_u_from_file((optind < argc) ?
-							    argv[optind++] :
-							    NULL);
+			csrfile = poptGetArg(pctx);
+			if (csrfile != NULL) {
+				csr = cm_submit_u_from_file(csrfile);
+			} else {
+				csr = getenv(CM_SUBMIT_CSR_ENV);
+				if (csr != NULL) {
+					csr = strdup(csr);
+				}
 			}
 			if ((csr == NULL) || (strlen(csr) == 0)) {
-				printf(_("Unable to read signing request.\n"));
-				help(argv[0]);
+				if (csrfile != NULL) {
+					printf(_("Unable to read signing request from file \"%s\".\n"),
+					       csrfile);
+				} else {
+					printf(_("Unable to read signing request from environment variable \"%s\".\n"),
+					       CM_SUBMIT_CSR_ENV);
+				}
+				poptPrintUsage(pctx, stdout, 0);
 				return CM_SUBMIT_STATUS_UNCONFIGURED;
 			}
 			csr = cm_submit_u_url_encode(csr);
@@ -448,7 +536,49 @@ main(int argc, char **argv)
 						 template,
 						 csr);
 		}
-		use_agent = FALSE;
+		/* Check for creds specified as options. */
+		for (j = 0; j < num_soptions; j++) {
+			if (strcmp(soptions[j].name, "uid") == 0) {
+				uid = NULL;
+			}
+			if (strcmp(soptions[j].name, "udn") == 0) {
+				udn = NULL;
+			}
+			if (strcmp(soptions[j].name, "pwd") == 0) {
+				pwd = NULL;
+			}
+			if (strcmp(soptions[j].name, "pin") == 0) {
+				pin = NULL;
+			}
+		}
+		/* Add client creds. */
+		if (uid != NULL) {
+			uid = cm_submit_u_url_encode(uid);
+			params = talloc_asprintf(ctx, "%s&uid=%s", params, uid);
+		}
+		if (udn != NULL) {
+			udn = cm_submit_u_url_encode(udn);
+			params = talloc_asprintf(ctx, "%s&udn=%s", params, udn);
+		}
+		if (pwd != NULL) {
+			pwd = cm_submit_u_url_encode(pwd);
+			params = talloc_asprintf(ctx, "%s&pwd=%s",
+						 params, pwd);
+		}
+		if (pin != NULL) {
+			pin = cm_submit_u_url_encode(pin);
+			params = talloc_asprintf(ctx, "%s&pin=%s",
+						 params, pin);
+		}
+		/* Add parameters specified on command line */
+		for (j = 0; j < num_soptions; j++) {
+			p = cm_submit_u_url_encode(soptions[j].name);
+			q = cm_submit_u_url_encode(soptions[j].value);
+			params = talloc_asprintf(ctx,
+						 "%s&%s=%s",
+						 params, p, q);
+		}
+		use_agent_approval = FALSE;
 		break;
 	case op_check:
 		/* Check if the certificate has been issued or rejected. */
@@ -457,9 +587,21 @@ main(int argc, char **argv)
 					 "%s&"
 					 "xml=true",
 					 params);
-		use_agent = FALSE;
+		use_agent_approval = FALSE;
 		break;
 	case op_approve:
+		if (agenturl == NULL) {
+			printf(_("No agent URL (-A) given, and no default "
+				 "known.\n"));
+			poptPrintUsage(pctx, stdout, 0);
+			return CM_SUBMIT_STATUS_UNCONFIGURED;
+		}
+		if ((sslcert == NULL) || (strlen(sslcert) == 0)) {
+			printf(_("No agent credentials (-n) given, but they "
+				 "are needed.\n"));
+			poptPrintUsage(pctx, stdout, 0);
+			return CM_SUBMIT_STATUS_UNCONFIGURED;
+		}
 		/* Reading profile defaults for this certificate, then applying
 		 * them and issuing a new certificate. */
 		url = talloc_asprintf(ctx, "%s/profileReview", agenturl);
@@ -472,7 +614,7 @@ main(int argc, char **argv)
 					  "%s&"
 					  "op=approve",
 					  params);
-		use_agent = TRUE;
+		use_agent_approval = TRUE;
 		break;
 	case op_retrieve:
 		/* Retrieving the new certificate. */
@@ -482,7 +624,20 @@ main(int argc, char **argv)
 					 "importCert=true&"
 					 "xml=true",
 					 params);
-		use_agent = FALSE;
+		use_agent_approval = FALSE;
+		break;
+	case op_profiles:
+		/* Retrieving the list of profiles. */
+		url = talloc_asprintf(ctx, "%s/profileList", eeurl);
+		if (strlen(params) > 0) {
+			params = talloc_asprintf(ctx,
+						 "%s&"
+						 "xml=true",
+						 params);
+		} else {
+			params = "xml=true";
+		}
+		use_agent_approval = FALSE;
 		break;
 	}
 
@@ -505,7 +660,7 @@ main(int argc, char **argv)
 					cainfo, capath, sslcert, sslkey, sslpin,
 					cm_submit_h_negotiate_off,
 					cm_submit_h_delegate_off,
-					use_agent ?
+					use_agent_approval || use_agent_submission ?
 					cm_submit_h_clientauth_on :
 					cm_submit_h_clientauth_off,
 					cm_submit_h_env_modify_off,
@@ -516,14 +671,14 @@ main(int argc, char **argv)
 		lastparams = params;
 		cm_submit_h_run(hctx);
 		if (verbose > 0) {
-			printf("%s \"%s?%s\"\n", "GET", url, params);
-			printf("code = %d\n", cm_submit_h_result_code(hctx));
-			printf("code_text = \"%s\"\n", cm_submit_h_result_code_text(hctx));
+			fprintf(stderr, "%s \"%s?%s\"\n", "GET", url, params);
+			fprintf(stderr, "code = %d\n", cm_submit_h_result_code(hctx));
+			fprintf(stderr, "code_text = \"%s\"\n", cm_submit_h_result_code_text(hctx));
 			syslog(LOG_DEBUG, "%s %s?%s\n", "GET", url, params);
 		}
-		results = cm_submit_h_results(hctx);
+		results = cm_submit_h_results(hctx, NULL);
 		if (verbose > 0) {
-			printf("results = \"%s\"\n", results);
+			fprintf(stderr, "results = \"%s\"\n", results);
 			syslog(LOG_DEBUG, "%s", results);
 		}
 		if (cm_submit_h_result_code(hctx) != 0) {
@@ -544,17 +699,49 @@ main(int argc, char **argv)
 			for (i = 0;
 			     (defaults != NULL) && (defaults[i] != NULL);
 			     i++) {
+				/* Check if this default is one of the
+				 * parameters we've been explicitly provided. */
+				for (j = 0; j < num_aoptions; j++) {
+					if (strcmp(defaults[i]->name,
+						   aoptions[j].name) == 0) {
+						break;
+					}
+				}
+				/* If we have a non-default value for it, skip
+				 * this default. */
+				if (j < num_aoptions) {
+					continue;
+				}
 				p = cm_submit_u_url_encode(defaults[i]->name);
 				q = cm_submit_u_url_encode(defaults[i]->value);
+				if (verbose > 0) {
+					fprintf(stderr, "setting \"%s\" to "
+						"default value \"%s\"\n",
+						p, q);
+				}
 				params2 = talloc_asprintf(ctx,
 							  "%s&%s=%s",
 							  params2, p, q);
 			};
+			/* Add parameters specified on command line */
+			for (j = 0; j < num_aoptions; j++) {
+				p = cm_submit_u_url_encode(aoptions[j].name);
+				q = cm_submit_u_url_encode(aoptions[j].value);
+				params2 = talloc_asprintf(ctx,
+							  "%s&%s=%s",
+							  params2, p, q);
+				if (verbose > 0) {
+					fprintf(stderr, "setting \"%s\" to "
+						"specified value \"%s\"\n",
+						p, q);
+				}
+			}
 			break;
 		case op_none:
 		case op_submit:
 		case op_check:
 		case op_retrieve:
+		case op_profiles:
 			/* No second form for these. */
 			break;
 		}
@@ -576,16 +763,19 @@ main(int argc, char **argv)
 			       cm_submit_h_result_code(hctx),
 			       lasturl);
 		}
+		talloc_free(ctx);
 		return CM_SUBMIT_STATUS_UNREACHABLE;
 	}
 	if (results == NULL) {
 		printf(_("Internal error: no response to \"%s?%s\".\n"),
 		       lasturl, lastparams);
+		talloc_free(ctx);
 		return CM_SUBMIT_STATUS_REJECTED;
 	}
 	switch (op) {
 	case op_none:
 		printf(_("Internal error: unknown state.\n"));
+		talloc_free(ctx);
 		return CM_SUBMIT_STATUS_UNCONFIGURED;
 		break;
 	case op_submit:
@@ -597,6 +787,7 @@ main(int argc, char **argv)
 		if (q != NULL) {
 			fprintf(stderr, "%s", q);
 		}
+		talloc_free(ctx);
 		return ret;
 		break;
 	case op_check:
@@ -608,6 +799,7 @@ main(int argc, char **argv)
 		if (q != NULL) {
 			fprintf(stderr, "%s", q);
 		}
+		talloc_free(ctx);
 		return ret;
 		break;
 	case op_approve:
@@ -620,6 +812,7 @@ main(int argc, char **argv)
 			if (q != NULL) {
 				fprintf(stderr, "%s", q);
 			}
+			talloc_free(ctx);
 			return ret;
 		} else {
 			ret = cm_submit_d_review_eval(ctx, results, lasturl,
@@ -630,6 +823,7 @@ main(int argc, char **argv)
 			if (q != NULL) {
 				fprintf(stderr, "%s", q);
 			}
+			talloc_free(ctx);
 			return ret;
 		}
 		break;
@@ -642,8 +836,22 @@ main(int argc, char **argv)
 		if (q != NULL) {
 			fprintf(stderr, "%s", q);
 		}
+		talloc_free(ctx);
+		return ret;
+		break;
+	case op_profiles:
+		ret = cm_submit_d_profiles_eval(ctx, results, lasturl,
+						can_agent, &p, &q);
+		if (p != NULL) {
+			fprintf(stdout, "%s", p);
+		}
+		if (q != NULL) {
+			fprintf(stderr, "%s", q);
+		}
+		talloc_free(ctx);
 		return ret;
 		break;
 	}
+	talloc_free(ctx);
 	return CM_SUBMIT_STATUS_UNCONFIGURED;
 }
