@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2011,2012,2013,2014,2015 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2012,2013,2014,2015,2016 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,11 @@
 #include <sys/stat.h>
 #include <errno.h>
 #include <fcntl.h>
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
 #include <limits.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -64,14 +68,14 @@
 
 #define OP_GET_CA_CAPS "GetCACaps"
 #define OP_GET_CA_CERT "GetCACert"
-#define OP_GET_CA_CHAIN "GetCAChain"
+#define OP_GET_CA_CERT_CHAIN "GetCACertChain"
 #define OP_GET_INITIAL_CERT "PKIOperation"
 #define OP_PKCSREQ "PKIOperation"
 enum known_ops {
 	op_unset,
 	op_get_ca_caps,
 	op_get_ca_certs,
-	op_get_initial_cert,
+	op_get_cert_initial,
 	op_pkcsreq,
 };
 
@@ -202,6 +206,7 @@ main(int argc, const char **argv)
 	int response_code = 0, response_code2 = 0;
 	enum known_ops op = op_unset;
 	const char *id = NULL, *cainfo = NULL;
+	char *poptarg;
 	char *message = NULL, *rekey_message = NULL;
 	const char *mode = NULL, *content_type = NULL, *content_type2 = NULL;
 	void *ctx;
@@ -260,7 +265,7 @@ main(int argc, const char **argv)
 			}
 		} else
 		if (strcasecmp(mode, CM_OP_POLL) == 0) {
-			op = op_get_initial_cert;
+			op = op_get_cert_initial;
 			message = getenv(CM_SUBMIT_SCEP_PKCSREQ_REKEY_ENV);
 			if (message == NULL) {
 				message = getenv(CM_SUBMIT_SCEP_PKCSREQ_ENV);
@@ -314,7 +319,7 @@ main(int argc, const char **argv)
 			op = op_get_ca_certs;
 			break;
 		case 'g':
-			op = op_get_initial_cert;
+			op = op_get_cert_initial;
 			break;
 		case 'p':
 			op = op_pkcsreq;
@@ -323,8 +328,9 @@ main(int argc, const char **argv)
 			racert = cm_submit_u_from_file(poptGetOptArg(pctx));
 			break;
 		case 'R':
-			cainfo = poptGetOptArg(pctx);
-			cacert = cm_submit_u_from_file(cainfo);
+			poptarg = poptGetOptArg(pctx);
+			cainfo = strdup(poptarg);
+			cacert = cm_submit_u_from_file(poptarg);
 			break;
 		case 'I':
 			certs = cm_submit_u_from_file(poptGetOptArg(pctx));
@@ -350,6 +356,7 @@ main(int argc, const char **argv)
 		printf(_("No SCEP operation (-c/-C/-g/-p) given, and no default known.\n"));
 		missing_args = TRUE;
 	}
+	id = cm_submit_u_url_encode(id);
 
 	/* Format the first (or only) HTTP request's parameters. */
 	switch (op) {
@@ -364,7 +371,7 @@ main(int argc, const char **argv)
 		/* First step: get the root certificate. */
 		params = talloc_asprintf(ctx, "operation=" OP_GET_CA_CERT "&message=%s", id);
 		break;
-	case op_get_initial_cert:
+	case op_get_cert_initial:
 		if ((racert == NULL) || (strlen(racert) == 0)) {
 			printf(_("No RA certificate (-r) given, and no default known.\n"));
 			missing_args = TRUE;
@@ -376,7 +383,7 @@ main(int argc, const char **argv)
 				}
 			}
 			if ((message == NULL) || (strlen(message) == 0)) {
-				printf(_("Error reading request, expected PKCS7 data.\n"));
+				printf(_("Error reading request.  Expected PKCS7 data containing a GetInitialCert pkiMessage, got nothing.\n"));
 				return CM_SUBMIT_STATUS_NEED_SCEP_MESSAGES;
 			}
 			/* First step: read capabilities for our use. */
@@ -395,7 +402,7 @@ main(int argc, const char **argv)
 				}
 			}
 			if ((message == NULL) || (strlen(message) == 0)) {
-				printf(_("Error reading request, expected PKCS7 data.\n"));
+				printf(_("Error reading request.  Expected PKCS7 data containing a PKCSReq pkiMessage, got nothing.\n"));
 				return CM_SUBMIT_STATUS_NEED_SCEP_MESSAGES;
 			}
 			/* First step: read capabilities for our use. */
@@ -518,9 +525,9 @@ main(int argc, const char **argv)
 		break;
 	case op_get_ca_certs:
 		/* Step two: request the chain. */
-		params2 = talloc_asprintf(ctx, "operation=" OP_GET_CA_CHAIN "&message=%s", id);
+		params2 = talloc_asprintf(ctx, "operation=" OP_GET_CA_CERT_CHAIN "&message=%s", id);
 		break;
-	case op_get_initial_cert:
+	case op_get_cert_initial:
 		/* Step two: actually poll.  If we have multiple messages which
 		 * we can use, decide which one to use. */
 		can_renewal = check_capability(results, results_length, "Renewal");
@@ -530,7 +537,7 @@ main(int argc, const char **argv)
 			tmp2 = message;
 		}
 		if ((tmp2 == NULL) || (strlen(tmp2) == 0)) {
-			printf(_("Error reading request, expected PKCS7 data.\n"));
+			printf(_("Error reading request.  Expected PKCS7 data containing a GetInitialCert pkiMessage, got nothing.\n"));
 			return CM_SUBMIT_STATUS_NEED_SCEP_MESSAGES;
 		} else
 		if (verbose > 0) {
@@ -555,7 +562,7 @@ main(int argc, const char **argv)
 			tmp2 = message;
 		}
 		if ((tmp2 == NULL) || (strlen(tmp2) == 0)) {
-			printf(_("Error reading request, expected PKCS7 data.\n"));
+			printf(_("Error reading request.  Expected PKCS7 data containing a PKCSReq pkiMessage, got nothing.\n"));
 			return CM_SUBMIT_STATUS_NEED_SCEP_MESSAGES;
 		} else
 		if (verbose > 0) {
@@ -643,7 +650,7 @@ main(int argc, const char **argv)
 			return CM_SUBMIT_STATUS_REJECTED;
 		}
 		break;
-	case op_get_initial_cert:
+	case op_get_cert_initial:
 	case op_pkcsreq:
 		/* ignore an error status */
 		break;
@@ -869,7 +876,7 @@ main(int argc, const char **argv)
 			return CM_SUBMIT_STATUS_UNREACHABLE;
 		}
 		break;
-	case op_get_initial_cert:
+	case op_get_cert_initial:
 	case op_pkcsreq:
 		if (strcasecmp(content_type2,
 			       "application/x-pki-message") == 0) {
@@ -970,12 +977,19 @@ main(int argc, const char **argv)
 				return CM_SUBMIT_STATUS_UNREACHABLE;
 			}
 			if (strcmp(pkistatus, SCEP_PKISTATUS_PENDING) == 0) {
+				if (verbose > 0) {
+					fprintf(stderr, "SCEP status is \"pending\".\n");
+					fprintf(stderr, "SCEP doesn't use cookie values, outputting server nonce.\n");
+				}
 				s = cm_store_base64_from_bin(ctx, sender_nonce,
 							     sender_nonce_length);
 				printf("%s\n", s);
 				return CM_SUBMIT_STATUS_WAIT;
 			} else
 			if (strcmp(pkistatus, SCEP_PKISTATUS_FAILURE) == 0) {
+				if (verbose > 0) {
+					fprintf(stderr, "SCEP status is \"failure\".\n");
+				}
 				if (failinfo == NULL) {
 					printf(_("Unspecified failure at server.\n"));
 				} else
@@ -1021,6 +1035,9 @@ main(int argc, const char **argv)
 				return CM_SUBMIT_STATUS_REJECTED;
 			} else
 			if (strcmp(pkistatus, SCEP_PKISTATUS_SUCCESS) == 0) {
+				if (verbose > 0) {
+					fprintf(stderr, "SCEP status is \"success\".\n");
+				}
 				u = payload;
 				p7 = d2i_PKCS7(NULL, &u, payload_length);
 				if (p7 == NULL) {
@@ -1093,6 +1110,9 @@ main(int argc, const char **argv)
 				free(s);
 				return CM_SUBMIT_STATUS_ISSUED;
 			} else {
+				if (verbose > 0) {
+					fprintf(stderr, "SCEP status is \"%s\".\n", pkistatus);
+				}
 				printf(_("Error: pkiStatus \"%s\" not recognized.\n"),
 				       pkistatus);
 				return CM_SUBMIT_STATUS_UNREACHABLE;
