@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2011,2012,2013,2014,2015 Red Hat, Inc.
+ * Copyright (C) 2009,2011,2012,2013,2014,2015,2017 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,9 @@
 #include <sys/wait.h>
 #include <errno.h>
 #include <fcntl.h>
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
 #include <limits.h>
 #include <stdint.h>
 #include <stdlib.h>
@@ -183,9 +186,19 @@ cm_submit_e_ready(struct cm_submit_state *state)
 				if ((msg != NULL) && (length > 0)) {
 					cm_log(1, "Child output:\n\"%.*s\"\n", length, msg);
 					/* If it's a single line, assume it's
-					 * log-worthy. */
+					 * log-worthy unless the code is all we
+					 * care about. */
 					if (strcspn(msg, "\n") >= (strlen(msg) - 2)) {
-						cm_log(0, "%s", msg);
+						switch (WEXITSTATUS(status)) {
+						case CM_SUBMIT_STATUS_ISSUED:
+						case CM_SUBMIT_STATUS_WAIT:
+						case CM_SUBMIT_STATUS_WAIT_WITH_DELAY:
+						case CM_SUB_STATUS_NEED_SCEP_DATA:
+							break;
+						default:
+							cm_log(0, "%s", msg);
+							break;
+						}
 					}
 					/* If it was an error, save it. */
 					if ((WEXITSTATUS(status) ==
@@ -824,6 +837,7 @@ maybe_setenv(const char *var, const char *val)
 	if ((var == NULL) || (val == NULL) || (strlen(val) == 0)) {
 		return -1;
 	}
+	cm_log(2, "Setting \"%s\" to \"%s\" for child.\n", var, val);
 	return setenv(var, val, 1);
 }
 
@@ -876,6 +890,7 @@ cm_submit_e_helper_main(int fd, struct cm_store_ca *ca,
 	maybe_setenv(CM_SUBMIT_COOKIE_ENV, entry->cm_ca_cookie);
 	maybe_setenv(CM_SUBMIT_CA_NICKNAME_ENV, entry->cm_ca_nickname);
 	maybe_setenv(CM_SUBMIT_PROFILE_ENV, entry->cm_template_profile);
+	maybe_setenv(CM_SUBMIT_ISSUER_ENV, entry->cm_template_issuer);
 	maybe_setenv(CM_SUBMIT_CERTIFICATE_ENV, entry->cm_cert);
 	/* Only pass SCEP data to the helper if we haven't used this set of
 	 * nonced data before.  It'll ask for fresh data if it needs it. */
@@ -924,6 +939,7 @@ cm_submit_e_helper_main(int fd, struct cm_store_ca *ca,
 		}
 		return -1;
 	}
+	cm_log(2, "Redirecting stdin and stderr to /dev/null, leaving stdout open for child \"%s\".\n", argv[0]);
 	cm_subproc_mark_most_cloexec(STDOUT_FILENO, -1, -1);
 	cm_log(1, "Running enrollment helper \"%s\".\n", argv[0]);
 	execvp(argv[0], argv);
