@@ -12,7 +12,7 @@ run_certutil -d "$tmpdir" -S -g $size -n keyi$size \
 	-x -t u -k dsa
 # Export the key.
 pk12util -d "$tmpdir" -o $size.p12 -W "" -n "keyi$size" > /dev/null 2>&1
-openssl pkcs12 -in $size.p12 -out key.$size -passin pass: -nodes -nocerts > /dev/null 2>&1
+openssl pkcs12 -in $size.p12 -out key.$size -passin pass: -nodes -nocerts > /dev/null 2>&1 | grep -v '^MAC verified OK$' || :
 # Read the public key and cache it.
 cat > entry.openssl.$size <<- EOF
 key_storage_type=FILE
